@@ -176,11 +176,21 @@ class ToolbarBuilder(object):
                 icon = kinds.icon_for_kind(entry.type)
             action.setIcon(icon)
         elif isinstance(entry, tools.ToolFolder):
-            # TODO: turn sub-folders into menus
-            toolbar.addSeparator()
+            # create menu
+            item = QtGui.QMenu(entry.title,toolbar)
+            menu_action = QtGui.QAction(entry.title, toolbar)
+            menu_action.setMenu(item)
+            if entry.icon:
+                icon = kinds.icon_by_resource_path(entry.icon)
+                menu_action.setIcon(icon)
+            toolbar.addAction(menu_action)
+            # change to button behaviour if top of menu
+            if isinstance(toolbar, QtGui.QToolBar):
+                button = toolbar.widgetForAction(menu_action)
+                button.setPopupMode(QtGui.QToolButton.InstantPopup)
+            # add menu options
             for child in entry:
-                self._add_tool_entry(child, toolbar, toolbars)
-            toolbar.addSeparator()
+                self._add_tool_entry(child, item, toolbars, folders)
         elif isinstance(entry, tools.ToolSet):
             # Lookup the folder named "toolset:...".
             # Note: project substitution is not supported here
@@ -539,8 +549,9 @@ class ToolBoxBuilder(object):
                     if icon is not None:
                         menu_action.setIcon(icon)
                     group.addAction(menu_action)
-                    button = group.widgetForAction(menu_action)
-                    button.setPopupMode(QtGui.QToolButton.InstantPopup)
+                    if isinstance(group, QtGui.QToolBar):
+                        button = group.widgetForAction(menu_action)
+                        button.setPopupMode(QtGui.QToolButton.InstantPopup)
             for child in entry:
                 self._add_tool_entry(child, item, groups, default_group_icon)
         elif isinstance(entry, tools.ToolSeparator):
