@@ -22,11 +22,12 @@ import bzrlib
 from bzrlib import (
     config,
     errors,
+    lazy_import,
     osutils,
     trace,
     urlutils,
     )
-from bzrlib.commands import shlex_split_unicode
+from bzrlib.cmdline import split
 from bzrlib.plugins import explorer
 from bzrlib.plugins.explorer.lib import (
     accessories_dialog,
@@ -73,16 +74,21 @@ except ImportError, e:
         'http://doc.bazaar.canonical.com/plugins/en/plugin-installation.html')
 if qbzr.version_info < (0, 17):
     trace.warning('Bazaar Explorer works better with QBzr 0.17 or later')
+import bzrlib.plugins.qbzr.lib.util
 from bzrlib.plugins.qbzr.lib.util import (
     open_browser,
     QBzrWindow,
     )
 
+lazy_import.lazy_import(globals(), """
+from bzrlib.plugins.explorer.lib.compatibility import send2trash
+""")
+
 
 _LP_PROJECT = "bzr-explorer"
 _HELP_URL = "http://doc.bazaar.canonical.com/explorer/en/documentation.html"
-_CODE_NAME = "Bartolomeu Dias"
-_CODE_NAME_URL = "http://en.wikipedia.org/wiki/Bartolomeu_Dias"
+_CODE_NAME = "Leif Ericson"
+_CODE_NAME_URL = "http://en.wikipedia.org/wiki/Leif_Ericson"
 
 
 _MSG_TYPE_TO_ICON = {
@@ -97,6 +103,7 @@ _AUTO_REFRESH_CMDS = [
     'add', 'commit',
     'pull', 'merge', 'update', 'push', 'send',
     'conflicts', 'switch', 'bind', 'unbind', 'revert', 'uncommit',
+    'shelve', 'unshelve',
     # This one really ought to be handled by a custom dialog so that the new
     # location can be opened. Refreshing the current page (often the
     # repository view) is better than nothing though.
@@ -113,7 +120,7 @@ def split_command_options(command):
         # Map xx.app -> "open -a /Applications/xx.app"
         if command.endswith(".app") and not command.startswith("open"):
             command = 'open -a "%s"' % command
-    return shlex_split_unicode(command)
+    return split(command)
 
 
 class QExplorerMainWindow(QBzrWindow):
@@ -138,6 +145,9 @@ class QExplorerMainWindow(QBzrWindow):
         self.ui = ui_explorer.Ui_MainWindow()
         self.ui.setupUi(self)
 
+        # setup refresher
+        self._refresher = _Refresher(self._do_refresh)
+
         # Setup some useful attributes
         self.app_name = gettext("Bazaar Explorer")
         self.desktop_env = desktop_env.DesktopEnvironment(desktop)
@@ -214,6 +224,12 @@ class QExplorerMainWindow(QBzrWindow):
 
         # Finish initialising the UI
         self.restoreSize("explorer", (800, 600))
+        # [bialix 2011/05/09] this is just dirty hack
+        # but I don't have any better workaround right now:
+        # qbzr config should not be cached inside explorer
+        # strictly to say it should not be cached at all
+        if getattr(bzrlib.plugins.qbzr.lib.util, '_qbzr_config', None) is not None:
+            bzrlib.plugins.qbzr.lib.util._qbzr_config = None
         self.init_status_bar()
         self.init_actions()
         self.init_history()
@@ -954,13 +970,11 @@ class QExplorerMainWindow(QBzrWindow):
                 # run it directly. Otherwise, run it via qrun
                 # making sure that we're passing --ui-mode.
                 if args[0][0] in ['q', 'g']:
-                    args = self._get_bzr_executable() + args
+                    args = ['bzr'] + args
                 elif tool.type == 'bzr-exec':
-                    args = self._get_bzr_executable () + \
-                           ['qrun', '--ui-mode', '--execute', '--'] + args
+                    args = ['bzr', 'qrun', '--ui-mode', '--execute', '--'] + args
                 else:
-                    args = self._get_bzr_executable() + \
-                           ['qrun', '--ui-mode', '--'] + args
+                    args = ['bzr', 'qrun', '--ui-mode', '--'] + args
             self.run_app(args, use_shell=(tool.type=='shell'))
 
     ## Show message helpers ##
@@ -1083,6 +1097,10 @@ class QExplorerMainWindow(QBzrWindow):
         :param use_shell: if True, cause the subprocess to use a shell
           to invoke args.
         """
+        # Replace 'bzr' with the executable that started this process
+        if args[0] == 'bzr':
+            args[:1] = self._get_bzr_executable()
+        
         # In diagnostic mode, just display what we would have done
         if self.dry_run:
             msg = u'Command: %s' % (args[0],)
@@ -1162,10 +1180,6 @@ class QExplorerMainWindow(QBzrWindow):
         # No luck - try as an external application
         args, post_action = self.get_command_from_app_suite(cmd_id, context)
         if args:
-            if args[0] == 'bzr':
-                # The config says to run a bzr command, so lets run the one
-                # that started this process
-                args[:1] = self._get_bzr_executable()
             refresh_on_success = post_action == 'refresh'
             self.run_app(args, refresh_on_success)
         else:
@@ -1311,10 +1325,31 @@ class QExplorerMainWindow(QBzrWindow):
             msg_template = gettext("Delete file %s?")
         msg = msg_template % (path,)
         if self.ask_question(title, msg):
-            deleter(path)
+            if self._preferences.get('delete-to-trash'):
+                self._delete_to_trash(path, deleter)
+            else:
+                deleter(path)
             if self._preferences.get('status-auto-refresh'):
                 self.do_refresh()
 
+    def _delete_to_trash(self, path, killer):
+        try:
+            send2trash(path)
+        except OSError, e:
+            try:
+                err = unicode(e)
+            except UnicodeError:
+                err = str(e)
+            msg = gettext(
+                """Can't delete "%(path)s" to trash.\n\n"""
+                "Details:\n"
+                "  %(error)s"
+                "\n\n"
+                "Delete it without sending to trash?"
+                ) % dict(path=path, error=err)
+            if self.ask_question(gettext("Error"), msg):
+                killer(path)
+
     def new_file(self, in_directory):
         if self.dry_run:
             msg = u'New File: %s' % (path,)
@@ -1334,12 +1369,8 @@ class QExplorerMainWindow(QBzrWindow):
                 why = gettext(unicode(ex))
                 msg = "%s %s." % (gettext(u"Unable to create file"), path)
                 self.show_error(title, msg, extra_info=why)
-        
+
     def new_directory(self, in_directory):
-        if self.dry_run:
-            msg = u'New Directory: %s' % (path,)
-            self.show_note("Diagnostic Mode", msg)
-            return
         title = gettext("New Directory")
         info = gettext("Destination directory: %s" % in_directory)
         label = gettext("Name of new directory:")
@@ -1347,6 +1378,10 @@ class QExplorerMainWindow(QBzrWindow):
         name = self.request_string(title, message)
         if name:
             path = osutils.pathjoin(in_directory, name)
+            if self.dry_run:
+                msg = u'New Directory: %s' % (path,)
+                self.show_note("Diagnostic Mode", msg)
+                return
             try:
                 os.mkdir(path)
             except OSError, ex:
@@ -1354,7 +1389,7 @@ class QExplorerMainWindow(QBzrWindow):
                 why = gettext(unicode(ex))
                 msg = "%s %s." % (gettext(u"Unable to create directory"), path)
                 self.show_error(title, msg, extra_info=why)
-            
+
     def _do_view_action(self, action, target, context=None):
         """Do an action on a target.
 
@@ -1557,7 +1592,6 @@ class QExplorerMainWindow(QBzrWindow):
     def _finish_switch(self):
         dialog = self._custom_modeless_dialogs['switch']
         self.do_refresh()
-        
 
     ## Toolbar helper methods ##
 
@@ -1725,11 +1759,16 @@ class QExplorerMainWindow(QBzrWindow):
         self.do_refresh(reopen=True)
 
     def do_refresh(self, reopen=False):
+        self._refresher.launch(reopen)  # shedule refresh in the background
+
+    def _do_refresh(self, reopen=False):
         self.location_viewer.refresh_current(reopen=reopen)
+        self.processEvents()
         # only refresh working tree if current view has one
         view = self.current_view()
         if view and view.has_wt():
             self._wt_browser.refresh()
+            self.processEvents()
         self.changed_view()
 
     def refresh_diagnostic_mode(self):
@@ -1925,6 +1964,7 @@ class QExplorerMainWindow(QBzrWindow):
             bookmark_stores, self.open_saved_location)
         history_browser = location_set_browser.HistoryBrowser(
             self._history_manager, self.open_saved_location)
+        self.location_viewer.locationAdded.connect(history_browser.refresh_view)
         location_sets = [
             (gettext("Bookmarks"), bookmarks_browser),
             (gettext("Recently Opened"), history_browser),
@@ -1961,9 +2001,10 @@ class QExplorerMainWindow(QBzrWindow):
             'explorer_version': version,
             'project_url': "http://doc.bazaar.canonical.com/explorer/en/",
             'using_information': using_info,
-            'purpose_text': gettext("Version Control for Human Beings"),
+            'purpose_text': ("Version Control for Human Beings"),
             'version_text': gettext("Version"),
             'copyright_text': gettext("Copyright"),
+            'ian_url': "http://bazaarvcs.wordpress.com/2010/09/01/farewell-ian/",
         }
         about_title = gettext("About %s" % (self.app_name,))
         about_info = (
@@ -1971,13 +2012,39 @@ class QExplorerMainWindow(QBzrWindow):
             u"%(version_text)s %(explorer_version)s<br>"
             u"<br>"
             u"<small>%(using_information)s</small><br>"
+            u"<hr />"
             u"<br>"
-            u"%(copyright_text)s \u00A9 2009 Canonical Ltd<br>"
+            u'Many thanks to <a href="%(ian_url)s">Ian Clatworthy</a> '
+              u'for his ideas and design.<br>'
+            u"<br>"
+            u"%(copyright_text)s \u00A9 2009-2011 Canonical Ltd<br>"
             u"<br>"
             u'<a href="%(project_url)s">%(project_url)s</a>' % tpl)
-        # Patch in the copyright year. (We do it this way
-        # for 1.0 because strings are frozen and changing the text
-        # will broken translations.)
-        this_year = '2010'  # TODO: Get from date lookup
-        about_info = about_info.replace('2009', '2009-%s' % (this_year,))
         QtGui.QMessageBox.about(self, about_title, about_info)
+
+
+class _Refresher(object):
+    """Internal lock-like object to avoid refreshing too much."""
+
+    def __init__(self, refresh_func):
+        self.sheduled = False
+        self.reopen = False
+        self.refresh_func = refresh_func
+
+    def launch(self, reopen=False):
+        if not self.sheduled:
+            self.sheduled = True
+            self.reopen = reopen
+            #print 'refresh sheduled (reopen=%s)' % reopen
+            QtCore.QTimer.singleShot(1, self._refresh)
+        else:
+            #print 'refresh is already sheduled'
+            pass
+
+    def _refresh(self):
+        #print 'refresh started'
+        try:
+            self.refresh_func(self.reopen)
+        finally:
+            self.sheduled = False
+        #print 'refresh finished'
