@@ -135,6 +135,14 @@ class _AbstractLocationModel(object):
     def group(self):
         """Return the group key or None if not part of a group."""
         return None
+    
+    def supports_wt(self):
+        """Return true if model should have a working tree"""
+        return False
+
+    def has_wt(self):
+        """Return true if model has a working tree"""
+        return False
 
 
 class ExtensibleLocationModel(_AbstractLocationModel):
@@ -275,6 +283,18 @@ class LocationModel(ExtensibleLocationModel):
         """
         return self.interesting_kind
 
+    def supports_wt(self):
+        """Return True if model should have a working tree"""
+        return (self.interesting_kind in [
+            BRANCH_KIND,
+            BOUND_BRANCH_KIND,
+            CHECKOUT_KIND,
+            ])
+
+    def has_wt(self):
+        """Return True if model has a working tree"""
+        return (self.supports_wt() and self.tree is not None)
+
     def category(self):
         """Return one of local, remote, None."""
         return {True: "local", False: "remote"}[self._local]
@@ -307,47 +327,38 @@ class LocationModel(ExtensibleLocationModel):
         """Return the kind and naming details."""
         # Get the pieces
         kind = self.kind()
-        repo_str = ""
-        br_str = ""
-        co_str = ""
-        br_base = None
+        locations = {
+                'this_location': None,
+                'master_location': None,
+                'repository': None,
+                }
         shared_tree_repo_found = False
+        def format_location(location):
+            return self.url_for_display(strip_trailing_slash(location))
         if kind == CHECKOUT_KIND:
-            co_base = self._root
-            co_str = os.path.basename(self.url_for_display(co_base))
-        if self.branch:
-            br_base = strip_trailing_slash(self.branch.base)
-            br_str = os.path.basename(self.url_for_display(br_base))
+            locations['this_location'] = format_location(self._root)
+            locations['master_location'] = format_location(self.branch.base)
+        elif kind == BOUND_BRANCH_KIND:
+            locations['this_location'] = format_location(self.branch.base)
+            locations['master_location'] = format_location(self.branch.get_bound_location())
+        elif kind == BRANCH_KIND:
+            locations['this_location'] = format_location(self.branch.base)
         if self.repository:
-            repo_base = strip_trailing_slash(
-                self.repository.bzrdir.root_transport.external_url())
-            if br_base is not None and repo_base == br_base:
-                loc_str = self.location
-                if loc_str.endswith(br_str):
-                    loc_str = os.path.basename(loc_str[:-(len(br_str) + 1)])
-                repo_str = '(%s)' % (loc_str,)
+            repository_location = format_location(self.repository.bzrdir.root_transport.external_url())
+            if repository_location != locations['this_location']:
+                locations['repository'] = repository_location
+                if os.path.basename(locations['repository']) == '.bzrbranches':
+                    locations['repository'] = os.path.split(locations['repository'])[0]
+                    shared_tree_repo_found = True
+                elif locations['repository'].endswith('/.bzr/branches'):
+                    locations['repository'] = locations['repository'][:-len('/.bzr/branches')]
+                    shared_tree_repo_found = True
+        for key in locations.keys():
+            if locations[key] is not None:
+                locations[key + '_base'] = os.path.basename(locations[key])
             else:
-                repo_path = self.url_for_display(repo_base)
-                repo_str = os.path.basename(repo_path)
-                # If this is a nested shared repo inside a checkout
-                # (as happens in the 'colocated-branches' model), use the
-                # checkout name as the label.
-                if repo_str == '.bzrbranches':
-                    repo_str = repo_path.split("/")[-2]
-                    shared_tree_repo_found = True
-                elif repo_path.endswith("/.bzr/branches"):
-                    repo_str = repo_path.split("/")[-3]
-                    shared_tree_repo_found = True
-                if kind != REPOSITORY_KIND:
-                    repo_str = '[%s]' % (repo_str,)
-        template = {
-            'repo': repo_str,
-            'br': br_str,
-            'co': co_str,
-            'relpath': self.relpath,
-            'pathsep': os.path.sep,
-        }
-        return kind, template, shared_tree_repo_found
+                locations[key + '_base'] = None
+        return kind, locations, shared_tree_repo_found
 
     def display_name(self):
         """Return a name suitable for displaying in a tab or title bar.
@@ -361,35 +372,31 @@ class LocationModel(ExtensibleLocationModel):
           [repository]
           checkout -> branch [repository]
         """
-        kind, template, shared_tree_repo_found = self._get_name_details()
-        if kind in [BRANCH_KIND, BOUND_BRANCH_KIND]:
-            if self.repository:
-                format = "%(br)s %(repo)s"
+        kind, locations, shared_tree_repo_found = self._get_name_details()
+        if kind == BRANCH_KIND:
+            if locations['repository'] is not None:
+                format = "%(this_location_base)s [%(repository_base)s]"
             else:
-                format = "%(br)s"
+                format = "%(this_location_base)s"
         elif kind == CHECKOUT_KIND:
-            if template['br'] == 'trunk':
-                format = "%(co)s"
-            else:
-                format = "%(co)s -> %(br)s"
-            if self.repository and not shared_tree_repo_found:
-                format += " %(repo)s"
+            format = "%(this_location_base)s -> %(master_location_base)s"
+            if locations['repository'] is not None and not shared_tree_repo_found:
+                format += " %(repository)s"
+        elif kind == BOUND_BRANCH_KIND:
+            format = "%(this_location_base)s -> %(master_location_base)s"
         elif kind == REPOSITORY_KIND:
-            format = "%(repo)s"
+            format = "[%(repository_base)s]"
         else:
             format = None
-        return format % template
+        return format % locations
 
     def display_local_name(self):
         """Return a name to display - context is implied. """
-        kind, template, _ = self._get_name_details()
-        if kind in [BRANCH_KIND, BOUND_BRANCH_KIND]:
-            format = "%(br)s"
-        elif kind == CHECKOUT_KIND:
-            if template['br'] == 'trunk':
-                format = "%(co)s"
-            else:
-                format = "%(co)s -> %(br)s"
+        kind, locations, _ = self._get_name_details()
+        if kind == BRANCH_KIND:
+            format = "%(this_location_base)s"
+        elif kind in [CHECKOUT_KIND, BOUND_BRANCH_KIND]:
+            format = "%(this_location_base)s -> %(master_location_base)s"
         elif kind == REPOSITORY_KIND:
             # Just the icon would do but it looks too plain if that's
             # the only object open. If the space could be better used
@@ -398,7 +405,25 @@ class LocationModel(ExtensibleLocationModel):
             format = gettext("Repository")
         else:
             format = None
-        return format % template
+        return format % locations
+
+    def display_tooltip(self):
+        kind, locations, shared_tree_repo_found = self._get_name_details()
+        format = kind + " "
+        if kind == BRANCH_KIND:
+            format += "%(this_location)s"
+            if locations['repository'] is not None:
+                format += "\n" + gettext("in repository") + " %(repository)s"
+        elif kind == CHECKOUT_KIND:
+            format += "%(this_location)s\n-> %(master_location)s"
+            if locations['repository'] is not None and not shared_tree_repo_found:
+                format += "\n" + gettext("in repository") + " %(repository)s"
+        elif kind == BOUND_BRANCH_KIND:
+            format += "%(this_location)s\n-> %(master_location)s"
+        elif kind == REPOSITORY_KIND:
+            format += "%(this_location)s"
+
+        return format % locations
 
     def display_diagnostics(self):
         """Return diagnostic information as a string.
