@@ -58,7 +58,8 @@ class _RepositoryViewBase(object):
         self._root_transport = None
         self._init_data()
         self._view = self._build_view()
-
+        self._init_watcher()
+        
     def _init_data(self):
         # When set, this is a tuple of ...
         # all, branches, bound_branches, checkouts, repositories
@@ -185,6 +186,24 @@ class _RepositoryViewBase(object):
                 QtCore.SIGNAL("selectionChanged(QItemSelection,QItemSelection)"),
                 self.do_selection_changed_on_all_tab)
 
+    def _init_watcher(self):
+        """create a QFileSystemWatcher to watch for changes in
+        the repository"""
+        root_transport = self._get_root_transport()
+        repo_url = root_transport.external_url()
+        if repo_url.startswith("file://"):
+            repo_path = urlutils.local_path_from_url(repo_url)
+            directory_watcher = QtCore.QFileSystemWatcher(
+                                       [QtCore.QString(repo_path)], self._view)
+            directory_watcher.directoryChanged.connect(
+                                        self._repo_directory_changed)
+
+    def _repo_directory_changed(self, directory):
+        """refresh view after short delay, called by the QFileSystemWatcher"""
+        # more reliable to delay refresh a bit else the external change
+        # may not have completed
+        QtCore.QTimer.singleShot(500, self.refresh_view)
+
     def do_open_location(self):
         abspath = self._get_abspath(self._relpath)
         self._action_callback("open-location", abspath)
@@ -240,7 +259,7 @@ class _RepositoryViewBase(object):
         return unicode(model.data(id_index).toString())
 
     def _get_abspath(self, relpath):
-        return self._get_root_transport().abspath(relpath)
+        return self._get_root_transport().abspath(urlutils.escape(relpath))
 
     def do_double_clicked(self, model_index):
         self._relpath = self._get_relpath_for_index(model_index)
