@@ -591,85 +591,25 @@ class _QBrowseFilterProxyModel(treewidget.TreeFilterProxyModel):
     def __init__(self, *args):
         treewidget.TreeFilterProxyModel.__init__(self, *args)
         self.text_to_match = None
-        self._interesting = None
-        self._all_data_loaded = False
-        self._text_match_count = 0
 
     def setTextToMatch(self, text):
         self.text_to_match = text
-        self._update_interesting()
         self.invalidateFilter()
 
-    def _update_interesting(self):
-        if not self.text_to_match:
-            self._interesting = None
-            return
-        text = self.text_to_match
-        interesting = set([''])
-        directories = set()
-        for path in self._iter_paths():
-            if path.find(text) >= 0:
-                interesting.add(path)
-                directories.update(parent_directories(path))
-        self._text_match_count = len(interesting)
-        interesting.update(directories)
-        #print "\ninteresting for %s ...\n%s\n" % (text,
-        #    "\n".join(sorted(interesting)))
-        self._interesting = interesting
-
-    def _iter_paths(self):
-        # The source model loads data lazily. We need everything loaded
-        # to find the complete set of paths though
-        self._ensure_all_data_loaded()
-        return iter(self.source_model.inventory_data_by_path.keys())
-
-    def _ensure_all_data_loaded(self):
-        if self._all_data_loaded:
-            return
-        model = self.source_model
-        for id in range(0, len(model.inventory_data)):
-            self._load_dirs(model, id)
-        self._all_data_loaded = True
-
-    def _load_dirs(self, model, id):
-        item_data = model.inventory_data[id]
-        if item_data.item.kind == "directory":
-            if item_data.children_ids is None:
-                # This locks and unlocks the tree each time.
-                # I wonder how that impacts performance?
-                model.load_dir(id)
-            for child_id in item_data.children_ids:
-                self._load_dirs(model, child_id)
-
-    def filterAcceptsRow(self, source_row, source_parent):
-        result = treewidget.TreeFilterProxyModel.filterAcceptsRow(
-            self, source_row, source_parent)
-        if result and self._interesting:
-            # Apply the text filter (matches found earlier)
-            path, kind = self._get_path_info(source_row, source_parent)
-            result = path in self._interesting
-        return result
-
-    def _get_path_info(self, source_row, source_parent):
-        """Return path, kind for a model item."""
-        model = self.source_model
-        parent_id = source_parent.internalId()
-        children_ids = model.inventory_data[parent_id].children_ids
-        # Why is this check required?
-        if len(children_ids) <= source_row:
-            return None, None
-        id = children_ids[source_row]
-        data = model.inventory_data[id]
-        return data.path, data.item.kind
-
-    def text_match_count(self):
-        """Number of items matching text or 0 if unknown.
-        
-        Note: The number is across the WHOLE tree, not just those displayed
-        after applying the combined filtering.
+    def filter_id(self, id, item_data):
+        """Determines wether a item should be displayed.
+        Returns :
+            * True: Show the item
+            * False: Donot show the item
+            * None: Show the item if there are any children that are visible.
         """
-        return self._text_match_count
-
+        show = treewidget.TreeFilterProxyModel.filter_id(self, id, item_data)
+        if self.text_to_match and show:
+            if item_data.path.find(self.text_to_match) >=1:
+                return True
+            else:
+                return None
+        return show
 
 class _QBrowseTreeWidget(treewidget.TreeWidget):
     """Subclass the standard QBzr TreeWidget to patch in more features.
@@ -781,9 +721,21 @@ class _QBrowseStyleBrowser(_AbstractBrowser):
             # Automatically expand the tree if the match count is small.
             # The number could be configurable one day provided the doc
             # was very clear about potential vs actual match counts
-            matches = self._tree_viewer.tree_filter_model.text_match_count()
-            if matches <= 10:
-                self._tree_viewer.expandAll()
+            parents = [QtCore.QModelIndex()] # start at the root
+            next_parents = []
+            expanded = 0
+            while parents and expanded <= 15:
+                for parent in parents:
+                    matches = self._tree_viewer.tree_filter_model.rowCount(parent)
+                    if matches and matches <= 5:
+                        self._tree_viewer.expand(parent)
+                    if (self._tree_viewer.isExpanded(parent) or
+                        parent == QtCore.QModelIndex()):
+                        expanded += matches
+                        for i in range(matches):
+                            next_parents.append(
+                                self._tree_viewer.tree_filter_model.index(i, 0, parent))
+                parents = next_parents
         # Reset our memory wrt what's expanded. This is perhaps a little
         # heavy handed wrt text matching as that isn't remembered across
         # tree switches. (Then again, perhaps it should be.)
