@@ -169,7 +169,7 @@ class _TupleListModel(QtGui.QStandardItemModel):
                         except (TypeError, ValueError):
                             item.setData(QVariant("Not displayable"),
                                          Qt.DisplayRole)
-                        item.setData(QVariant(tuple_list[i][j]))
+                        item.setData(QVariant(tuple_list[i][j] or ''))
                     else:
                         item = QtGui.QStandardItem()
                         try:
@@ -185,7 +185,7 @@ class _TupleListModel(QtGui.QStandardItemModel):
                     self.setItem(i,j,item)
         else:
             self._column_count = 0
-        
+
     def raw_data(self):
         return self._raw_data
 
